﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using TPLSamples.Utility;

namespace TPLSamples.Models
{
    public class AggregateExceptionCatchSample : Sample
    {
        //used by DrawGreyScale to set the number of rows we started
        private int rowsStarted = 0;

        public override string SampleName
        {
            get { return "Aggregate Exception - Catch Exceptions Sample"; }
        }

        public override bool ImageRequired
        {
            get { return true; }
        }

        public override void Run(System.Drawing.Bitmap bmp = null, Action<string> UpdateLog = null)
        {
            Stopwatch s = new Stopwatch();
            s.Start();

            try
            {
                rowsStarted = 0;
                DrawGreyScale(bmp, UpdateLog);
            }
            catch (AggregateException ae)
            {
                UpdateLog("Started " + rowsStarted + " rows.");

                ae.Handle((x) =>
                {
                    UpdateLog("Handling an exception.");
                    return true;
                });
            }

            s.Stop();
            RunTime = s.Elapsed;
        }

        private void DrawGreyScale(System.Drawing.Bitmap bmp, Action<string> UpdateLog)
        {
            ConcurrentQueue<Exception> exceptions = new ConcurrentQueue<Exception>();
            System.Drawing.Imaging.BitmapData bmData = bmp.LockBits(new System.Drawing.Rectangle(0, 0, bmp.Width, bmp.Height), System.Drawing.Imaging.ImageLockMode.ReadWrite, System.Drawing.Imaging.PixelFormat.Format24bppRgb);
            int stride = bmData.Stride;
            System.IntPtr Scan0 = bmData.Scan0;
            unsafe
            {
                byte* start = (byte*)(void*)Scan0;

                int height = bmp.Height;
                int width = bmp.Width;

                Parallel.For(0, height, y =>
                {
                    try
                    {
                        UpdateLog("Starting line " + y.ToString());
                        Interlocked.Increment(ref rowsStarted);
                        byte* p = start + (y * stride);
                        for (int x = 0; x < width; ++x)
                        {
                            byte blue = p[0];
                            byte green = p[1];
                            byte red = p[2];

                            p[0] = p[1] = p[2] = (byte)(.299 * red
                                + .587 * green
                                + .114 * blue);

                            p += 3;
                        }

                        if (y >= 100)
                        {
                            UpdateLog("Throwing an exception at " + y);
                            if (y % 2 == 0)
                                throw new StackOverflowException("yeah, we got a stack overflow.");
                            else
                                throw new ArgumentNullException("yeah, we got a null argument.");
                        }
                    }
                    catch (StackOverflowException)
                    {
                        UpdateLog("Internally handled the StackOverflowException.");
                    }
                    catch (Exception e)
                    {
                        exceptions.Enqueue(e);
                    }
                });
            }
            bmp.UnlockBits(bmData);

            if (exceptions.Count > 0)
                throw new AggregateException(exceptions);
        }
    }
}
