﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace TPLSamples.Models
{
	public class GreyScaleParallelSample : Sample
	{
		public override string SampleName
		{
			get { return "Grey Scale Parallel Sample"; }
		}

		public override bool ImageRequired
		{
			get { return true; }
		}

		public override void Run(System.Drawing.Bitmap bmp = null, Action<string> UpdateLog = null)
		{
			if(bmp == null)
				throw new InvalidOperationException("Bitmap must be defined.");
			
			Stopwatch s = new Stopwatch();
			s.Start();

			System.Drawing.Imaging.BitmapData bmData = bmp.LockBits(new System.Drawing.Rectangle(0, 0, bmp.Width, bmp.Height), System.Drawing.Imaging.ImageLockMode.ReadWrite, System.Drawing.Imaging.PixelFormat.Format24bppRgb);
			int stride = bmData.Stride;
			System.IntPtr Scan0 = bmData.Scan0;
			unsafe
			{
				byte* start = (byte*)(void*)Scan0;

				int height = bmp.Height;
				int width = bmp.Width;

				Parallel.For(0, height, y =>
				{
					byte* p = start + (y * stride);
					for (int x = 0; x < width; ++x)
					{
						byte blue = p[0];
						byte green = p[1];
						byte red = p[2];

						p[0] = p[1] = p[2] = (byte)(.299 * red
							+ .587 * green
							+ .114 * blue);

						p += 3;
					}
				});
			}
			bmp.UnlockBits(bmData);

			s.Stop();
			RunTime = s.Elapsed;
		}
	}
}
